set class GsNMethod

set class GsNMethod
category: 'Debugging Support'
method:
_setBreakAtIp: ipOffset operation: opcFlags frame: fpOffset process: aGsProcess

"Set breakpoint at specified ipOffset.  ipOffset is an instruction offset,
 which is a absolute byte offset within the receiver.

 Checks that the receiver is not part of the implementation of the scheduler.

 opcFlags is a SmallInteger , fields have these masks
    opcode 		   16rFF
    includeMethodStartBcs 16r100 , a Boolean , only affects opcode==1
        when ipOffset == -1 .

 opcode  action
   0	 set or reenable method breakpoint
   1     set single step breakpoint (takes precedence over method break)
   2     delete method breakpoint or disabled method breakpoint
   3     delete single step breakpoint
   4     disable method breakpoint, no action if breakpoint not set

  If ipOffset == -1, then apply the action to all step points within the
  method.  

  For opcode 1 only, to restrict breakpoint to a specific frame 
  and/or GsProcess , fpOffset and/or aGsProcess must be non-nil. "

(Processor _scheduler _criticalMethods includes: self) ifTrue:[
  ^ Error signal:'cannot set breakpoints within the ProcessorScheduler'
].
^ self __setBreakAtIp: ipOffset operation: opcFlags frame: fpOffset process: aGsProcess
%

category: 'Repository Conversion'
method:  
recompile

"Recompiles the method for execution in a Gs64 v3.0 or later system.
 Returns a GsNMethod, or signals a CompileError or CompileWarning .
 
 See also  recompileIntoMethodDict:intoCategories: "

self methodCompilerVersion >= 4 ifTrue:[ ^ self "recompile not needed"].
^ self recompileIntoMethodDict: nil intoCategories: nil
%

! edited for 42285
category: 'Repository Conversion'
method: 
recompileIntoMethodDict: aMethodDict intoCategories: aCategDict

"Recompiles the method for execution in a Gs64 v3.0 or later system.

 Literal variables whose key is in ObsoleteClasses
 are replaced by the appropriate association from the ObsoleteClassesDict .
 Other literal variables are looked up in the literal pool of the receiver,
 before searching class variables, class pool dictionaries, or 
 the current symbolList.  Thus recompilation should work without knowing
 what symbolList was used when the receiver was created.   

 The result is a GsNMethod if compilation succeeds, otherwise
 an error is generated.  environmentId zero is used for all compilations.

 If aMethodDict is not nil, and the compilation succeeds,
 the resulting method is added to aMethodDict instead of to
 the receiver's method dictionary.  This is used to add methods
 to per-session dictionaries.

 If aMethodDict is not nil and aCategDict is not nil and
 the compilation succeeds, the resulting method is added aCategDict
 instead of the receiver's categories.

 If the receiver is an anonymous method, the sender of this method is
 reponsible for saving the result."

| cls litVars symAssocCls obsDict newSrc |
litVars := { }  . 
symAssocCls := SymbolAssociation .
obsDict := self class _classVars at: #ObsoleteClassesDict .
self literals do: [:aLit |
  (aLit isKindOf: symAssocCls) ifTrue:[ | litName aVal newLit srcName lookupKey obsArr |
    litName := aLit key .
    (obsArr := obsDict at: litName otherwise: nil) ifNotNil:[ 
      aVal := aLit value .
      (aVal ~~ nil and:[ aVal == (obsArr at:1)]) ifTrue:[ 
         srcName := obsArr at: 2 . 
         lookupKey := obsArr at: 3 .
         newLit := Globals associationAt: lookupKey otherwise: nil .
      ].
    ].
    newLit ifNotNil:[ litVars add: srcName ; add: newLit ] 
           ifNil:[    litVars add: litName ; add: aLit ].
  ] ifFalse:[
    "handle compiler optimization of using value of invariant assocations
     for some classes."
    (aLit isClass and:[ aLit isMeta not ]) ifTrue:[ | cname |
      (Globals associationAt: (cname := aLit name) otherwise: nil) ifNotNil:[:assoc |
         assoc _value == aLit ifTrue:[   
           litVars add: cname ; add: assoc . 
        ]
      ]
    ]
  ]
].
SessionTemps current at:#OldLitVars put: litVars .
newSrc := self sourceString .
(System gemConfigurationAt:#GemConvertArrayBuilder) ifTrue:[
  newSrc := GsNMethod convertArrayBuildersInString: newSrc .
].
cls := self inClass .
cls ifNil:[ 
  ^ newSrc _compileInContext: nil symbolList: nil oldLitVars: litVars
		environmentId: 0
] ifNotNil:[ | categ |
 categ := cls categoryOfSelector: self selector . 
  ^ cls _checkCompileResult: ( cls _primitiveCompileMethod: newSrc symbolList: nil 
         category: categ oldLitVars: litVars 
         intoMethodDict: aMethodDict intoCategories: aCategDict
         intoPragmas: nil environmentId: 0 )
       source: newSrc .
]
%

